package WWW::Bizowie::API::Response;

use strict;
use warnings;

use Mo;

=head1 NAME

WWW::Bizowie::API::Response - Response from the Bizowie API

=head1 METHODS

=head2 success

=head2 data

=cut

has success => ();
has data    => ();

=head1 AUTHORS

L<Bizowie|http://bizowie.com/>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2013 Bizowie

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.

=cut

1;

